@@ -2,20 +2,20 @@
 ;;; un-define.el --- fundamental template to comprise UTF
 ;;;                  and Unicode conversion on Mule-UCS.
 
-;; Copyright (C) 1999 Miyashita Hisashi
+;; Copyright (C) 1999-2000 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, 
 ;;           character set, coding-system, ISO/IEC 10646,
 ;;           Unicode, UTF-8, UTF-16
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -32,6 +32,15 @@
 ;;;   This module provide such facilities, that is APIs for
 ;;;   other applications to use Unicode, UTF, and so on.
 
+;;; Autoload section
+(autoload 'ucs-to-char "unicode")
+(autoload 'char-to-ucs "unicode")
+(autoload 'tae-modify-translation "tae")
+(autoload 'insert-ucs-character "unicode"
+  "Insert character which is converted from give UCS codepoint."
+  t)
+
+;;; Required for compile, and also persistent section.
 (eval-and-compile
   (defvar un-define-basic-translation-charset-order-list
     '(ascii
@@ -60,23 +69,42 @@
       katakana-jisx0201
       )
     "*Charset order used by un-define translation")
+
+  (defun un-define-get-translation-from-charset (charset)
+    (intern (format "unicode-%s-translation-rule"
+		    (symbol-name charset))))
 ;;
 ;; Configuration functions for convenience.
 ;;
 
   (defun un-define-decode-template (tr read write)
-    (require 'mucs)
+    (require 'unicode)
     (mucs-ccl-stream-form
      (mucs-ccl-read 'ucs-generic read)
      (tae-compile tr t)
      (mucs-ccl-write write)))
 
   (defun un-define-encode-template (tr read write)
-    (require 'mucs)
+    (require 'unicode)
     (mucs-ccl-stream-form
      (mucs-ccl-read 'char-packed-1 read)
      (tae-compile tr nil)
-     (mucs-ccl-write write)))
+     (cond ((eq write 'utf-7)
+	    (mucs-ccl-utf-7-encode
+	     'utf-7-direct-or-optional-character-p-translation
+	     nil))
+	   ((eq write 'utf-7-dos)
+	    (mucs-ccl-utf-7-encode
+	     'utf-7-direct-or-optional-character-p-translation
+	     t))
+	   ((eq write 'utf-7-safe)
+	    (mucs-ccl-utf-7-encode
+	     'utf-7-direct-character-p-translation nil))
+	   ((eq write 'utf-7-safe-dos)
+	    (mucs-ccl-utf-7-encode
+	     'utf-7-direct-character-p-translation t))
+	   (t
+	    (mucs-ccl-write write)))))
 
   (defun un-define-get-coding-system-alist
     (coding-category &optional mime-charset charsets)
@@ -96,45 +124,24 @@
   ;; predefined configuration
   ;; Any settings in here affect only
   ;; byte-compiled file.
-
   (require 'unicode)
-  (require 'trans-util) ;; for char-codepoint ;_;
-
-  ;; load-table
-  (mapcar
-   (lambda (x)
-     (require-unicode-charset-data x))
-   un-define-basic-translation-charset-order-list)
 
+  ;; load-table & declare basic translation rule.
   ;; translation definition start
-
-  ;; basic translation rule, which can be modified dynamically.
+  ;;; basic translation rule, which can be modified dynamically.
   (tae-declare-translation
    'unicode-basic-translation-rule
-   `(| ,us-ascii-vs-unicode-assoc
-       ,iso-8859-1-vs-unicode-assoc
-       ,iso-8859-2-vs-unicode-assoc
-       ,iso-8859-3-vs-unicode-assoc
-       ,iso-8859-4-vs-unicode-assoc
-       ,iso-8859-5-vs-unicode-assoc
-;       ,iso-8859-6-vs-unicode-assoc
-       ,iso-8859-7-vs-unicode-assoc
-       ,iso-8859-8-vs-unicode-assoc
-       ,iso-8859-9-vs-unicode-assoc
-       ,ipa-vs-unicode-assoc
-       ,jis-x-0208-1990-vs-unicode-assoc
-       ,jis-x-0212-vs-unicode-assoc
-       ,gb2312-80-vs-unicode-assoc
-       ,cns11643-1-vs-unicode-assoc
-       ,cns11643-2-vs-unicode-assoc
-       ,cns11643-3-vs-unicode-assoc
-       ,cns11643-4-vs-unicode-assoc
-       ,cns11643-5-vs-unicode-assoc
-       ,cns11643-6-vs-unicode-assoc
-       ,cns11643-7-vs-unicode-assoc
-       ,ks-c-5601-1987-vs-unicode-assoc
-       ,jis-x-0201-latin-vs-unicode-assoc
-       ,jis-x-0201-katakana-vs-unicode-assoc)
+   `(| ,@(mapcar
+	  (lambda (charset)
+	    (let ((sym
+		   (un-define-get-translation-from-charset
+		    charset)))
+	      (require-unicode-charset-data charset)
+	      (tae-declare-translation
+	       sym
+	       (symbol-value (get charset 'unicode-assoc)))
+	      sym))
+	  un-define-basic-translation-charset-order-list))
    t) ;; for dynamic modification!
 
   ;; below translations are static.
@@ -377,6 +384,76 @@
      'unicode-line-separator-translation-rule
      'utf-16-be 'emacs-mule))))
 
+; UTF-7 conversions
+
+ (mucs-define-conversion
+  utf-7-unix-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule 'emacs-mule 'utf-7))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-safe-unix-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-7-safe))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-unix-stream-decoder
+  stream
+  (utf-7-decode-buffer-magnification
+   ((un-define-decode-template
+     'unicode-basic-stream-translation-rule 'utf-7 'emacs-mule))))
+
+ (mucs-define-conversion
+  utf-7-dos-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule 'emacs-mule 'utf-7-dos))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-safe-dos-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-7-safe-dos))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-dos-stream-decoder
+  stream
+  (utf-7-decode-buffer-magnification
+   ((un-define-decode-template
+     'unicode-basic-stream-translation-rule 'utf-7 'emacs-mule-dos))))
+
+ (mucs-define-conversion
+  utf-7-mac-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-mac-line-separator-translation-rule
+     'emacs-mule 'utf-7))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-safe-mac-stream-encoder
+  stream
+  (utf-7-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-mac-line-separator-translation-rule
+     'emacs-mule 'utf-7-safe))
+   ((mucs-ccl-utf-7-encode-eof))))
+ (mucs-define-conversion
+  utf-7-mac-stream-decoder
+  stream
+  (utf-7-decode-buffer-magnification
+   ((un-define-decode-template
+     'unicode-mac-line-separator-translation-rule
+     'utf-7 'emacs-mule))))
+
 ;;; For non-stream conversion.
 
  (mucs-define-conversion
@@ -505,6 +582,58 @@
      ,(un-define-get-coding-system-alist
        'coding-category-utf-16-be)
      [utf-16-be-unix utf-16-be-dos utf-16-be-mac]))
+
+    (utf-7
+     (utf-7-unix
+      ?7 "UTF-7 coding system.  Output Set O characters directly."
+      utf-7-unix-stream-decoder utf-7-unix-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-ccl 'utf-7)
+      unix)
+     (utf-7-dos
+      ?7 "UTF-7 coding system.  Output Set O characters directly."
+      utf-7-dos-stream-decoder utf-7-dos-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-ccl 'utf-7)
+      dos)
+     (utf-7-mac
+      ?7 "UTF-7 coding system.  Output Set O characters directly."
+      utf-7-mac-stream-decoder utf-7-mac-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-ccl 'utf-7)
+      mac)
+     (utf-7
+      ?7 "UTF-7 coding system.  Output Set O characters directly."
+      utf-7-unix-stream-decoder utf-7-unix-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-ccl 'utf-7)
+      [utf-7-unix utf-7-dos utf-7-mac]))
+
+    (utf-7-safe
+     (utf-7-safe-unix
+      ?7 "UTF-7 coding system.  Shift Set O characters."
+      utf-7-unix-stream-decoder utf-7-safe-unix-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-ccl 'utf-7)
+      unix)
+    (utf-7-safe-dos
+     ?7 "UTF-7 coding system.  Shift Set O characters."
+     utf-7-dos-stream-decoder utf-7-safe-dos-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-ccl 'utf-7)
+     dos)
+    (utf-7-safe-mac
+     ?7 "UTF-7 coding system.  Shift Set O characters."
+     utf-7-mac-stream-decoder utf-7-safe-mac-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-ccl 'utf-7)
+     mac)
+    (utf-7-safe
+     ?7 "UTF-7 coding system.  Shift Set O characters."
+     utf-7-unix-stream-decoder utf-7-safe-unix-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-ccl 'utf-7)
+     [utf-7-safe-unix utf-7-safe-dos utf-7-safe-mac]))
     ))
 
 ) ;; un-define package definition ends here
@@ -529,26 +658,21 @@
        category-list)))
 
 ;
-; Unicode or its transformation format support function.
+; Charset order dynamic modification. (Very simple version!)
 ;
 
-(defun ucs-to-char (codepoint)
-  (require 'unicode)
-  (mucs-convert 'ucs-codepoint-to-emacs-char-conversion
-		codepoint))
-
-(defun char-to-ucs (char)
-  (require 'unicode)
-  (logior (lsh (charset-id (char-charset char)) 16)
-	  (char-codepoint char))
-  (mucs-convert 'emacs-char-to-ucs-codepoint-conversion
-		char))
-
-(defun insert-ucs-character (codepoint)
-  (interactive "nUCS codepoint:")
-  (insert (or (ucs-to-char codepoint)
-	      (error "Invalid or cannot translate:U+%X"
-		     codepoint))))
+(defun un-define-change-charset-order (&optional order-list)
+  "Change conversion priority on un-define-basic-translation-rule.
+ORDER-LIST must be a list that consists of charsets, which specifies
+an order of conversion between internal characters and UCS characters."
+  (if (null order-list)
+      (setq order-list
+	    un-define-basic-translation-charset-order-list))
+  (tae-modify-translation
+   'unicode-basic-translation-rule
+   `(| ,@(mapcar
+	  (function un-define-get-translation-from-charset)
+	  order-list))))
 
 (provide 'un-define)
 
